/**
 * @file display.h
 * @author CuBeatSystems
 * @author Shinichiro Nakamura
 */

/*
 *  TOPPERS Realtime System Sample Program - LPCXpresso GPS Clock
 *
 *  Copyright (C) 2010-2013 CuBeatSystems
 *  Copyright (C) 2010-2013 Shinichiro Nakamura
 *
 *  上記著作権者は，以下の(1)〜(4)の条件を満たす場合に限り，本ソフトウェ
 *  ア（本ソフトウェアを改変したものを含む．以下同じ）を使用・複製・改
 *  変・再配布（以下，利用と呼ぶ）することを無償で許諾する．
 *  (1) 本ソフトウェアをソースコードの形で利用する場合には，上記の著作
 *      権表示，この利用条件および下記の無保証規定が，そのままの形でソー
 *      スコード中に含まれていること．
 *  (2) 本ソフトウェアを，ライブラリ形式など，他のソフトウェア開発に使
 *      用できる形で再配布する場合には，再配布に伴うドキュメント（利用
 *      者マニュアルなど）に，上記の著作権表示，この利用条件および下記
 *      の無保証規定を掲載すること．
 *  (3) 本ソフトウェアを，機器に組み込むなど，他のソフトウェア開発に使
 *      用できない形で再配布する場合には，次のいずれかの条件を満たすこ
 *      と．
 *    (a) 再配布に伴うドキュメント（利用者マニュアルなど）に，上記の著
 *        作権表示，この利用条件および下記の無保証規定を掲載すること．
 *    (b) 再配布の形態を，別に定める方法によって，TOPPERSプロジェクトに
 *        報告すること．
 *  (4) 本ソフトウェアの利用により直接的または間接的に生じるいかなる損
 *      害からも，上記著作権者およびTOPPERSプロジェクトを免責すること．
 *      また，本ソフトウェアのユーザまたはエンドユーザからのいかなる理
 *      由に基づく請求からも，上記著作権者およびTOPPERSプロジェクトを
 *      免責すること．
 *
 *  本ソフトウェアは，無保証で提供されているものである．上記著作権者お
 *  よびTOPPERSプロジェクトは，本ソフトウェアに関して，特定の使用目的
 *  に対する適合性も含めて，いかなる保証も行わない．また，本ソフトウェ
 *  アの利用により直接的または間接的に生じたいかなる損害に関しても，そ
 *  の責任を負わない．
 */

#ifndef DISPLAY_H
#define DISPLAY_H

#ifdef __cplusplus
extern "C" {
#endif

#define SEGSEL_A (1 << 0) /**< セグメント選択ビットマップ。 */
#define SEGSEL_B (1 << 1) /**< セグメント選択ビットマップ。 */
#define SEGSEL_C (1 << 2) /**< セグメント選択ビットマップ。 */
#define SEGSEL_D (1 << 3) /**< セグメント選択ビットマップ。 */
#define SEGSEL_E (1 << 4) /**< セグメント選択ビットマップ。 */
#define SEGSEL_F (1 << 5) /**< セグメント選択ビットマップ。 */
#define SEGSEL_G (1 << 6) /**< セグメント選択ビットマップ。 */
#define SEGSEL_P (1 << 7) /**< セグメント選択ビットマップ。 */

#define SEGSEL_NUM0 (SEGSEL_A | SEGSEL_B | SEGSEL_C | SEGSEL_D | SEGSEL_E | SEGSEL_F)               /**< セグメント選択ビットマップ。 */
#define SEGSEL_NUM1 (SEGSEL_B | SEGSEL_C)                                                           /**< セグメント選択ビットマップ。 */
#define SEGSEL_NUM2 (SEGSEL_A | SEGSEL_B | SEGSEL_G | SEGSEL_E | SEGSEL_D)                          /**< セグメント選択ビットマップ。 */
#define SEGSEL_NUM3 (SEGSEL_A | SEGSEL_B | SEGSEL_G | SEGSEL_C | SEGSEL_D)                          /**< セグメント選択ビットマップ。 */
#define SEGSEL_NUM4 (SEGSEL_F | SEGSEL_G | SEGSEL_B | SEGSEL_C)                                     /**< セグメント選択ビットマップ。 */
#define SEGSEL_NUM5 (SEGSEL_A | SEGSEL_F | SEGSEL_G | SEGSEL_C | SEGSEL_D)                          /**< セグメント選択ビットマップ。 */
#define SEGSEL_NUM6 (SEGSEL_A | SEGSEL_F | SEGSEL_E | SEGSEL_D | SEGSEL_C | SEGSEL_G)               /**< セグメント選択ビットマップ。 */
#define SEGSEL_NUM7 (SEGSEL_A | SEGSEL_B | SEGSEL_C)                                                /**< セグメント選択ビットマップ。 */
#define SEGSEL_NUM8 (SEGSEL_A | SEGSEL_B | SEGSEL_C | SEGSEL_D | SEGSEL_E | SEGSEL_F | SEGSEL_G)    /**< セグメント選択ビットマップ。 */
#define SEGSEL_NUM9 (SEGSEL_A | SEGSEL_B | SEGSEL_C | SEGSEL_D | SEGSEL_F | SEGSEL_G)               /**< セグメント選択ビットマップ。 */

#define SEGSEL_DOT_LEFT_UPPER           (1 << 0) /**< ドット選択。 */
#define SEGSEL_DOT_LEFT_LOWER           (1 << 1) /**< ドット選択。 */
#define SEGSEL_DOT_CENTER_LEFT_UPPER    (1 << 2) /**< ドット選択。 */
#define SEGSEL_DOT_CENTER_LEFT_LOWER    (1 << 3) /**< ドット選択。 */
#define SEGSEL_DOT_CENTER_RIGHT_UPPER   (1 << 4) /**< ドット選択。 */
#define SEGSEL_DOT_CENTER_RIGHT_LOWER   (1 << 5) /**< ドット選択。 */
#define SEGSEL_DOT_RIGHT_UPPER          (1 << 6) /**< ドット選択。 */
#define SEGSEL_DOT_RIGHT_LOWER          (1 << 7) /**< ドット選択。 */

typedef unsigned char segsel_t;

/**
 * @brief LPCXpresso Clock制御用構造体。
 * @details
 * 本来は実装の詳細をCソースコードに隠蔽し、ここでは不完全定義としたいところだ。
 * しかし、小規模組み込みシステム（特にマイコン）において、
 * ハンドラ領域を確保するための標準的なメモリアロケーションサービスが使える事を
 * 前提にしない方が移植性を高める結果を生むことがある。
 * ここでは完全定義として実装し、ユーザ側で制御構造体の領域を確保してもらう。
 */
typedef struct {
    unsigned char counter;          /**< 表示カウンタ。 */
    unsigned char colsel;           /**< カラム選択番号。 */
    segsel_t segsel[8];             /**< セグメント選択ビットマップ。 */
    unsigned char brightness[8];    /**< 輝度。 */
} display_t;

void display_open(display_t *p);
void display_7seg_write(
        display_t *p,
        const unsigned char segsel0,
        const unsigned char segsel1,
        const unsigned char segsel2,
        const unsigned char segsel3,
        const unsigned char segsel4,
        const unsigned char segsel5,
        const segsel_t dotsel);
void display_7seg_brightness(
        display_t *p,
        const int index,
        const unsigned char brightness);
void display_7seg_clock(display_t *p);
void display_close(display_t *p);

#ifdef __cplusplus
}
#endif

#endif

